@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.45 2005/02/02 14:33:38 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.3 2005/07/18 01:42:48 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -48,8 +48,6 @@
 
 #ifdef NONE_BLOCK
 static void set_nonblock(int fd);
-#endif
-#ifdef NOT_USED
 static void unset_nonblock(int fd);
 #endif
 
@@ -80,6 +78,7 @@ void do_child(int unix_fd, int inet_fd)
 	struct timezone tz;
 	int child_idle_sec;
 	struct timeval timeout;
+	static int connected;
 
 	pool_debug("I am %d", getpid());
 
@@ -138,7 +137,7 @@ void do_child(int unix_fd, int inet_fd)
 		if (frontend == NULL)
 		{
 			/* check select() timeout */
-			if (pool_config.child_life_time > 0 &&
+			if (connected && pool_config.child_life_time > 0 &&
 				timeout.tv_sec == 0 && timeout.tv_usec == 0)
 			{
 				pool_debug("child life %d seconds expired", pool_config.child_life_time);
@@ -148,9 +147,11 @@ void do_child(int unix_fd, int inet_fd)
 			continue;
 		}
 
-#ifdef NOT_USED
 		/* set frontend fd to blocking */
 		unset_nonblock(frontend->fd);
+
+#ifdef NOT_USED
+		set_nonblock(frontend->fd);
 #endif
 
 		/* set busy flag and clear child idle timer */
@@ -269,6 +270,8 @@ void do_child(int unix_fd, int inet_fd)
 		if (pool_config.replication_strict == 0)
 			pool_enable_timeout();
 
+		connected = 1;
+
 		/* query process loop */
 		for (;;)
 		{
@@ -381,7 +384,6 @@ static void set_nonblock(int fd)
 }
 #endif
 
-#ifdef NOT_USED
 /*
  * unset non-block flag
  */
@@ -402,7 +404,6 @@ static void unset_nonblock(int fd)
 		exit(1);
 	}
 }
-#endif
 
 /*
 * perform accept() and return new fd
@@ -832,11 +833,16 @@ static RETSIGTYPE die(int sig)
 {
 	exit_request = 1;
 
+	pool_debug("child receives shutdown request signal %d", sig);
+
 	switch (sig)
 	{
 		case SIGTERM:	/* smart shutdown */
 			if (idle == 0)
+			{
+				pool_debug("child receives smart shutdown request but it's not in idle state");
 				return;
+			}
 
 		case SIGINT:	/* fast shutdown */
 		case SIGQUIT:	/* immediate shutdown */
